"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeCreateTokenAccountIxs = exports.makeCreateMintIxs = exports.makeRequestCuIx = exports.makeCreateMarketAccountsIxs = void 0;
const web3_js_1 = require("@solana/web3.js");
const Aaob = __importStar(require("@chugach-foundation/aaob"));
const buffer_layout_1 = require("buffer-layout");
const serum_1 = require("@project-serum/serum");
const makeCreateMarketAccountsIxs = async (client) => {
    const eventCapacity = 50;
    const callbackInfoLen = 34;
    const orderCapacity = 1000;
    const signers = [];
    const ixs = [];
    // Market account
    const marketFull = new web3_js_1.Keypair();
    ixs.push(web3_js_1.SystemProgram.createAccount({
        fromPubkey: client.walletPubkey,
        lamports: await client.connection.getMinimumBalanceForRentExemption(Aaob.MarketState.LEN),
        newAccountPubkey: marketFull.publicKey,
        programId: client.cypherPID,
        space: Aaob.MarketState.LEN
    }));
    signers.push(marketFull);
    // Event queue account
    const eventQueueFull = new web3_js_1.Keypair();
    const eventQueueSize = Aaob.EventQueue.computeAllocationSize(eventCapacity, callbackInfoLen);
    ixs.push(web3_js_1.SystemProgram.createAccount({
        fromPubkey: client.walletPubkey,
        lamports: await client.connection.getMinimumBalanceForRentExemption(eventQueueSize),
        newAccountPubkey: eventQueueFull.publicKey,
        programId: client.cypherPID,
        space: eventQueueSize
    }));
    signers.push(eventQueueFull);
    // Bids account
    const bidsFull = new web3_js_1.Keypair();
    const slabSize = Aaob.Slab.computeAllocationSize(orderCapacity, callbackInfoLen);
    ixs.push(web3_js_1.SystemProgram.createAccount({
        fromPubkey: client.walletPubkey,
        lamports: await client.connection.getMinimumBalanceForRentExemption(slabSize),
        newAccountPubkey: bidsFull.publicKey,
        programId: client.cypherPID,
        space: slabSize
    }));
    signers.push(bidsFull);
    // Asks account
    const asksFull = new web3_js_1.Keypair();
    ixs.push(web3_js_1.SystemProgram.createAccount({
        fromPubkey: client.walletPubkey,
        lamports: await client.connection.getMinimumBalanceForRentExemption(slabSize),
        newAccountPubkey: asksFull.publicKey,
        programId: client.cypherPID,
        space: slabSize
    }));
    signers.push(asksFull);
    const priceHistoryFull = new web3_js_1.Keypair();
    ixs.push(await client.accounts.priceHistory.createInstruction(priceHistoryFull));
    signers.push(priceHistoryFull);
    return {
        orderbook: marketFull.publicKey,
        eventQueue: eventQueueFull.publicKey,
        bids: bidsFull.publicKey,
        asks: asksFull.publicKey,
        priceHistory: priceHistoryFull.publicKey,
        ixs,
        signers
    };
};
exports.makeCreateMarketAccountsIxs = makeCreateMarketAccountsIxs;
const makeRequestCuIx = (payer, units) => {
    const p = new web3_js_1.PublicKey('ComputeBudget111111111111111111111111111111');
    const params = { instruction: 0, units, additionalFee: 0 };
    const layout = (0, buffer_layout_1.struct)([
        (0, buffer_layout_1.u8)('instruction'),
        (0, buffer_layout_1.u32)('units'),
        (0, buffer_layout_1.u32)('additionalFee')
    ]);
    const data = Buffer.alloc(layout.span);
    layout.encode(params, data);
    const keys = [
        {
            pubkey: payer,
            isSigner: true,
            isWritable: false
        }
    ];
    return new web3_js_1.TransactionInstruction({
        keys,
        programId: p,
        data
    });
};
exports.makeRequestCuIx = makeRequestCuIx;
const makeCreateMintIxs = async (provider, authority, mint, decimals) => {
    return [
        web3_js_1.SystemProgram.createAccount({
            fromPubkey: provider.wallet.publicKey,
            newAccountPubkey: mint,
            space: 82,
            lamports: await provider.connection.getMinimumBalanceForRentExemption(82),
            programId: serum_1.TokenInstructions.TOKEN_PROGRAM_ID
        }),
        serum_1.TokenInstructions.initializeMint({
            mint,
            decimals: decimals ?? 0,
            mintAuthority: authority
        })
    ];
};
exports.makeCreateMintIxs = makeCreateMintIxs;
const makeCreateTokenAccountIxs = async (provider, newAccountPubkey, mint, owner, lamports) => {
    if (lamports === undefined) {
        lamports = await provider.connection.getMinimumBalanceForRentExemption(165);
    }
    return [
        web3_js_1.SystemProgram.createAccount({
            fromPubkey: provider.wallet.publicKey,
            newAccountPubkey,
            space: 165,
            lamports,
            programId: serum_1.TokenInstructions.TOKEN_PROGRAM_ID
        }),
        serum_1.TokenInstructions.initializeAccount({
            account: newAccountPubkey,
            mint,
            owner
        })
    ];
};
exports.makeCreateTokenAccountIxs = makeCreateTokenAccountIxs;
//# sourceMappingURL=utils.js.map